@@ -3,12 +3,41 @@ from collections.abc import MutableMapping
 from configparser import ConfigParser, NoOptionError
 import os
 import logging
+import re
+
 
 class OppParser(ConfigParser):
 
+    def __init__(self, **kwargs):
+        kwargs.setdefault("interpolation", None)
+        kwargs.setdefault("inline_comment_prefixes", "#")
+        super().__init__(**kwargs)
+
     def optionxform(self, optionstr):
         return optionstr
 
+    @classmethod
+    def resolve_includes(cls, ini_path, output_path=None, encoding="utf-8"):
+        """
+        Read and resolve include directives in ini_path and return string representation.
+        This will work on the string representation only and wil thus keep comment intact.
+        If output_path is given save result as new file
+        """
+        opp = cls()
+        file_content = opp.create_temp_file_with_includes([ini_path])[0]
+
+        if output_path is None:
+            return file_content
+
+        # check if output_path is dir and add file name if it is.
+        if os.path.isdir(output_path):
+            output_path = os.path.join(output_path,
+                                       os.path.basename(ini_path))
+
+        with open(output_path, "w") as f:
+            f.writelines(file_content)
+        return  file_content
+
     def read(self, filenames, encoding=None):
 
         if isinstance(filenames, (str, bytes, os.PathLike)):
@@ -54,10 +83,23 @@ class OppParser(ConfigParser):
         else:
             return True
 
+    def write_sections(self, fp, sections, space_around_delimiters=True):
+        """Write an .ini-format representation of the configuration state.
+
+        If `space_around_delimiters' is True (the default), delimiters
+        between keys and values are surrounded by spaces.
+        """
+        if space_around_delimiters:
+            d = " {} ".format(self._delimiters[0])
+        else:
+            d = self._delimiters[0]
+
+        for section in sections:
+            self._write_section(fp, section,
+                                self._sections[section].items(), d)
 
     def get_file_content_recursively(self, f_):
 
-
         logging.info(f" Relative path found: {f_}")
         f = os.path.join(os.path.abspath(f_))
         logging.info(f"Generate absolute path: {f_} ")
@@ -83,7 +125,6 @@ class OppParser(ConfigParser):
             print(f"{f} has file include: {self.has_file_include(f)} ({inc_file_name})")
 
             if self.has_file_include(f):
-
                 file_content = file_content.replace(
                     replace_string, self.get_file_content_recursively(inc_file_name)
                 )
@@ -169,28 +210,31 @@ class OppConfigFileBase(MutableMapping):
 
     @classmethod
     def from_path(
-        cls, ini_path, config, cfg_type=OppConfigType.EDIT_LOCAL, is_parent=False
+            cls, ini_path, config, cfg_type=OppConfigType.EDIT_LOCAL, is_parent=False
     ):
-        _root = OppParser(inline_comment_prefixes="#")
+        _root = OppParser()
         _root.read(ini_path)
+        _base_dir = os.path.dirname(ini_path)
 
-        return cls(_root, config, cfg_type, is_parent)
+        return cls(_root, config, _base_dir, cfg_type, is_parent)
 
     def __init__(
-        self,
-        root_cfg: OppParser,
-        config_name: str,
-        cfg_type=OppConfigType.EDIT_LOCAL,
-        is_parent=False,
+            self,
+            root_cfg: OppParser,
+            config_name: str,
+            _base_dir: str,
+            cfg_type=OppConfigType.EDIT_LOCAL,
+            is_parent=False,
     ):
         self._root: OppParser = root_cfg
         self._cfg_type = cfg_type
         self._sec = self._ensure_config_prefix(config_name)
+        self._base_dir = _base_dir
 
         if not self._has_section_(self._sec):
             raise ValueError(f"no section found with name {self._sec}")
         self._parent_cfg = []
-        self.section_hierarchy = [self._ensure_config_prefix(self._sec)]
+        self._section_hierarchy = [self._ensure_config_prefix(self._sec)]
         self._is_parent = is_parent
         if not self._is_parent:
             stack = [iter(self.parents)]
@@ -198,9 +242,9 @@ class OppConfigFileBase(MutableMapping):
                 for p in stack[0]:
                     if p == "":
                         continue
-                    _pp = OppConfigFileBase(self._root, p, is_parent=True)
+                    _pp = OppConfigFileBase(self._root, p, self._base_dir, is_parent=True)
                     self._parent_cfg.append(_pp)
-                    self.section_hierarchy.append(self._ensure_config_prefix(p))
+                    self._section_hierarchy.append(self._ensure_config_prefix(p))
                     if len(_pp.parents) > 0:
                         stack.append(iter(_pp.parents))
                 else:
@@ -208,17 +252,43 @@ class OppConfigFileBase(MutableMapping):
 
         if config_name != "General" and self._has_section_("General"):
             self._parent_cfg.append(
-                OppConfigFileBase(self._root, "General", is_parent=True)
+                OppConfigFileBase(self._root, "General", self._base_dir, is_parent=True)
             )
             self.section_hierarchy.append("General")
 
+    @property
+    def base_path(self):
+        return self._base_dir
+
+    @property
+    def all_sections(self):
+        return self._root.sections()
+
+    @property
+    def section_hierarchy(self):
+        return self._section_hierarchy
+
+    def resolve_path(self, key):
+        val =  self.__getitem__(key)
+        p = re.compile('absFilePath\("(.*)"\)')
+        m = p.match(val)
+        if m:
+            return os.path.join(self.base_path, m.group(1))
+        else:
+            return os.path.join(self.base_path, val.strip('"'))
+
+
     def read(self):
         self.read()
         pass
 
-    def writer(self, fp):
+    def writer(self, fp, selected_config_only=False):
         """ write the current state to the given file descriptor. Caller must close file."""
-        self._root.write(fp)
+        if selected_config_only:
+            # write current section hierarchy with General first (reverse list)
+            self._root.write_sections(fp, self.section_hierarchy[::-1])
+        else:
+            self._root.write(fp)
 
     @staticmethod
     def _ensure_config_prefix(val):
@@ -363,12 +433,12 @@ class OppConfigFileBase(MutableMapping):
 
     def __len__(self) -> int:
         _len = 0
-        for s in self.section_hierarchy:
+        for s in self._section_hierarchy:
             _len += len(self._root.items(s))
         return _len
 
     def __iter__(self):
-        for s in self.section_hierarchy:
+        for s in self._section_hierarchy:
             for item in self._root.items(s):
                 yield item
 
@@ -408,3 +478,11 @@ class OppConfigFile(OppConfigFileBase):
 
     def __init__(self, root_cfg: OppParser, config_name: str):
         super().__init__(root_cfg, config_name)
+
+
+if __name__ == "__main__":
+
+    OppParser.resolve_includes(
+        "/home/vm-sts/repos/crownet/crownet/simulations/mucFreiheitLte/omnetpp.ini",
+        output_path= "/home/vm-sts/repos/crownet/crownet/simulations/mucFreiheitLte/omnetpp_merged.ini"
+    )
\ No newline at end of file
